import { __extends, __read, __spread } from "tslib";
import { EOrientation } from './interfaces';
import { ALayoutContainer, withChanged } from './ALayoutContainer';
import { ASequentialLayoutContainer, wrap } from './ASequentialLayoutContainer';
import { LayoutContainerEvents } from '../';
var SplitLayoutContainer = /** @class */ (function (_super) {
    __extends(SplitLayoutContainer, _super);
    function SplitLayoutContainer(document, options, ratio, child1, child2) {
        var _this = _super.call(this, document, options) || this;
        _this.minChildCount = 2;
        _this.type = 'split';
        _this._ratios = [];
        console.assert(ratio === undefined || (ratio >= 0 && ratio <= 1));
        _this.node.dataset.layout = 'split';
        if (!_this.options.fixed) {
            _this.node.addEventListener('mousedown', function (evt) {
                if (_this.isSeparator(evt.target)) {
                    //dragging
                    var index = Math.floor(Array.from(_this.node.children).indexOf(evt.target) / 2);
                    _this.enableDragging(index);
                }
            });
        }
        if (ratio !== undefined) {
            console.assert(child1 != null && child2 != null);
            _this.push(child1, -1, 1);
            _this.push(child2, -1, (1 - ratio) / ratio);
        }
        return _this;
    }
    SplitLayoutContainer.prototype.defaultOptions = function () {
        return Object.assign(_super.prototype.defaultOptions.call(this));
    };
    SplitLayoutContainer.prototype.place = function (child, reference, area) {
        console.assert(area !== 'center');
        var index = this._children.indexOf(reference) + (area === 'right' || area === 'bottom' ? 1 : 0);
        return this.push(child, index, 0.5);
    };
    SplitLayoutContainer.prototype.isSeparator = function (elem) {
        return elem.parentElement === this.node && elem.dataset.layout === 'separator';
    };
    SplitLayoutContainer.prototype.enableDragging = function (index) {
        var _this = this;
        var bak = this._ratios.slice();
        var mouseMove = function (evt) {
            var n = _this.node;
            var bb = n.getBoundingClientRect();
            var x = evt.clientX - bb.left - n.clientLeft + n.scrollLeft;
            var y = evt.clientY - bb.top - n.clientTop + n.scrollTop;
            var ratio = _this.options.orientation === EOrientation.HORIZONTAL ? x / n.offsetWidth : y / n.offsetHeight;
            _this.setRatioImpl(index, ratio);
            //no events
            evt.stopPropagation();
            evt.preventDefault();
        };
        var disable = function (evt) {
            if (evt.target !== evt.currentTarget && evt.type === 'mouseleave') {
                return;
            }
            _this.node.classList.remove('slider-dragging');
            _this.node.removeEventListener('mousemove', mouseMove);
            _this.node.removeEventListener('mouseup', disable);
            _this.node.removeEventListener('mouseleave', disable);
            var act = _this._ratios.slice();
            if (!bak.every(function (b, i) { return act[i] === b; })) {
                //changed fire event just once
                _this.fire(withChanged(LayoutContainerEvents.EVENT_CHANGE_SPLIT_RATIOS), bak, act);
            }
        };
        this.node.classList.add('slider-dragging');
        this.node.addEventListener('mousemove', mouseMove);
        this.node.addEventListener('mouseup', disable);
        this.node.addEventListener('mouseleave', disable);
    };
    SplitLayoutContainer.prototype.setRatio = function (index, ratio) {
        var bak = this._ratios.slice();
        this.setRatioImpl(index, ratio);
        this.fire(withChanged(LayoutContainerEvents.EVENT_CHANGE_SPLIT_RATIOS), bak, this._ratios.slice());
    };
    SplitLayoutContainer.prototype.setRatioImpl = function (index, ratio) {
        var _this = this;
        console.assert(ratio >= 0 && ratio <= 1);
        var bak = this._ratios.slice();
        if (index === 0 || index >= (this.length - 2)) {
            if (index > 0 && index === (this.length - 2)) {
                // easier to manipulate the last then the 2nd last
                index += 1;
                ratio = 1 - ratio;
            }
            //corner cases
            var old = this._ratios[index];
            var others = this._ratios.reduce(function (a, r) { return a + r; }, -old);
            if (others > 0) {
                var factor_1 = (others + (old - ratio)) / others;
                this._ratios.forEach(function (r, i) { return _this._ratios[i] = r * factor_1; });
            }
            else {
                //even
                this._ratios.forEach(function (r, i) { return _this._ratios[i] = (1 - ratio) / (_this._ratios.length - 1); });
            }
            this._ratios[index] = ratio;
            this.updateRatios();
            return;
        }
        //we want that the left sum is our ratio
        var left = this._ratios.slice(0, index + 1);
        var before = left.reduce(function (a, r) { return a + r; }, 0);
        var factorBefore = ratio / before;
        if (factorBefore > 0) {
            left.forEach(function (r, i) { return _this._ratios[i] = r * factorBefore; });
        }
        else {
            left.forEach(function (r, i) { return _this._ratios[i] = (1 - ratio) / left.length; });
        }
        var right = this._ratios.slice(index + 1);
        var after = right.reduce(function (a, r) { return a + r; }, 0);
        var factorAfter = (1 - ratio) / after;
        if (factorAfter > 0) {
            right.forEach(function (r, i) { return _this._ratios[i + index + 1] = r * factorAfter; });
        }
        else {
            right.forEach(function (r, i) { return _this._ratios[i + index + 1] = ratio / right.length; });
        }
        this.updateRatios();
    };
    SplitLayoutContainer.prototype.squeeze = function (separator, dir) {
        var index = Math.floor(Array.from(this.node.children).indexOf(separator) / 2);
        this.setRatio(index + (dir === 'right' ? 1 : 0), 0);
    };
    SplitLayoutContainer.prototype.updateRatios = function () {
        var _this = this;
        var sum = this._ratios.reduce(function (a, r) { return a + r; }, 0);
        this._ratios.forEach(function (r, i) { return _this._ratios[i] = r / sum; }); //normalize
        var act = this._ratios.map(function (r) { return Math.round(r * 100); });
        this.forEach(function (c, i) {
            var wrapper = c.node.parentElement.style;
            wrapper.flex = act[i] + " " + act[i] + " 0";
            wrapper.display = act[i] <= 1 ? 'none' : null;
            c.resized();
        });
    };
    Object.defineProperty(SplitLayoutContainer.prototype, "ratios", {
        get: function () {
            return this._ratios.slice();
        },
        set: function (values) {
            var _a;
            console.assert(values.length === this._ratios.length);
            (_a = this._ratios).splice.apply(_a, __spread([0, this._ratios.length], values));
            this.updateRatios();
        },
        enumerable: true,
        configurable: true
    });
    SplitLayoutContainer.prototype.getPadding = function () {
        return (this.length - 1) * SplitLayoutContainer.SEPARATOR_WIDTH;
    };
    SplitLayoutContainer.prototype.push = function (child, index, ratio) {
        if (index === void 0) { index = -1; }
        if (ratio === void 0) { ratio = 0; }
        var r = _super.prototype.push.call(this, child, index);
        if (index < 0 || index >= (this._children.length - 1)) {
            this._ratios.push(ratio);
        }
        else {
            //assume we are in the replace mode and compute the missing ratio
            this._ratios.splice(index, 0, ratio);
        }
        this.updateRatios();
        return r;
    };
    SplitLayoutContainer.prototype.addedChild = function (child, index) {
        var _this = this;
        _super.prototype.addedChild.call(this, child, index);
        if (index < 0 || index >= (this.length - 1)) {
            //+1 since we already changed the children
            this.node.appendChild(wrap(child));
        }
        else if (index === 0) {
            //assume we are in the replace mode
            this.node.insertBefore(wrap(child), this.node.firstChild);
        }
        else {
            //assume we are in the replace mode -> consider separator
            this.node.insertBefore(wrap(child), this._children[index + 1].node.parentElement.previousSibling);
        }
        if (this.length > 1) {
            this.node.insertAdjacentHTML('beforeend', SplitLayoutContainer.SEPARATOR);
            var separator_1 = this.node.lastElementChild;
            if (!this.options.fixed) {
                separator_1.firstElementChild.addEventListener('click', function (evt) {
                    evt.preventDefault();
                    evt.stopPropagation();
                    _this.squeeze(separator_1, 'left');
                });
                separator_1.lastElementChild.addEventListener('click', function (evt) {
                    evt.preventDefault();
                    evt.stopPropagation();
                    _this.squeeze(separator_1, 'right');
                });
            }
            else {
                separator_1.classList.add('fixed');
            }
            if (index > 0) {
                this.node.insertBefore(separator_1, child.node.parentElement);
            }
            else {
                this.node.insertBefore(separator_1, child.node.parentElement.nextSibling);
            }
        }
        child.visible = this.visible;
    };
    SplitLayoutContainer.prototype.replace = function (child, replacement) {
        var index = this._children.indexOf(child);
        console.assert(index >= 0);
        var ratio = this._ratios[index];
        this.takeDownChild(child);
        this.setupChild(replacement);
        this._children.splice(index, 1, replacement);
        this.addedChild(replacement, index);
        this.updateRatios();
        return true;
    };
    SplitLayoutContainer.prototype.takeDownChild = function (child) {
        var wrapper = child.node.parentElement;
        //in case of the first one use the next one since the next child is going to be the first one
        var separator = wrapper.previousElementSibling || wrapper.nextElementSibling;
        if (separator) {
            separator.remove();
        }
        wrapper.remove();
        _super.prototype.takeDownChild.call(this, child);
    };
    SplitLayoutContainer.prototype.remove = function (child) {
        var index = this._children.indexOf(child);
        this._ratios.splice(index, 1);
        var r = _super.prototype.remove.call(this, child);
        this.updateRatios();
        return r;
    };
    SplitLayoutContainer.prototype.persist = function () {
        return Object.assign(_super.prototype.persist.call(this), {
            type: 'split',
            ratios: this.ratios.map(function (r) { return Math.round(r * 100) / 100; }),
            fixedLayout: this.options.fixedLayout
        });
    };
    SplitLayoutContainer.restore = function (dump, restore, doc) {
        console.assert(dump.children.length >= 2);
        var ratios = dump.ratios;
        var options = Object.assign(ALayoutContainer.restoreOptions(dump), {
            orientation: EOrientation[dump.orientation],
            fixedLayout: dump.fixedLayout === true
        });
        var r = new SplitLayoutContainer(doc, options, ratios[0], restore(dump.children[0]), restore(dump.children[1]));
        dump.children.slice(2).forEach(function (d, i) { return r.push(restore(d), ratios[i + 2]); });
        //force specific ratios
        r.ratios = ratios;
        return r;
    };
    SplitLayoutContainer.derive = function (node, derive) {
        var children = Array.from(node.children);
        console.assert(children.length >= 2);
        var deriveOrientation = function () {
            if (node.dataset.layout.startsWith('v') || (node.dataset.orientation && node.dataset.orientation.startsWith('v'))) {
                return EOrientation.VERTICAL;
            }
            return EOrientation.HORIZONTAL;
        };
        var deriveRatios = function () {
            var ratio = node.dataset.ratio ? parseFloat(node.dataset.ratio) : NaN;
            if (!isNaN(ratio)) {
                var rest = 1 - ratio;
                var r_1 = [ratio];
                for (var i = 1; i < children.length; ++i) {
                    r_1.push(rest / (children.length - 1));
                }
                return r_1;
            }
            var ratios = node.dataset.ratios ? node.dataset.ratios.split(' ').map(function (d) { return parseFloat(d); }) : [];
            if (ratios.every(function (d) { return !isNaN(d); })) {
                if (ratios.length < children.length) {
                    var sum = ratios.reduce(function (a, b) { return a + b; }, 0);
                    var missing = children.length - ratios.length;
                    for (var i = 0; i < missing; ++i) {
                        ratios.push((1 - sum) / missing);
                    }
                }
                return ratios.slice(0, children.length);
            }
            //generate uniform
            return children.map(function (_) { return 1 / children.length; });
        };
        var options = Object.assign(ALayoutContainer.deriveOptions(node), {
            orientation: deriveOrientation(),
            fixedLayout: Boolean(node.dataset.fixedLayout)
        });
        var ratios = deriveRatios();
        var r = new SplitLayoutContainer(node.ownerDocument, options);
        children.forEach(function (c) { return r.push(derive(c)); });
        r.ratios = ratios;
        return r;
    };
    SplitLayoutContainer.SEPARATOR = "<div data-layout=\"separator\"><span title=\"Squeeze Left\"></span><span title=\"Squeeze Right\"></span></div>";
    SplitLayoutContainer.SEPARATOR_WIDTH = 5;
    return SplitLayoutContainer;
}(ASequentialLayoutContainer));
export default SplitLayoutContainer;
//# sourceMappingURL=SplitLayoutContainer.js.map