import { __extends } from "tslib";
import { AParentLayoutContainer } from './AParentLayoutContainer';
import { ALayoutContainer, withChanged } from './ALayoutContainer';
import { dropAble } from 'phovea_core/src';
import { LayoutContainerEvents } from '../';
var TabbingLayoutContainer = /** @class */ (function (_super) {
    __extends(TabbingLayoutContainer, _super);
    function TabbingLayoutContainer(document, options) {
        var children = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            children[_i - 2] = arguments[_i];
        }
        var _this = _super.call(this, document, options) || this;
        _this.minChildCount = 0;
        _this.type = 'tabbing';
        _this.mouseEnter = function () { return _this.header.classList.add('show-header'); }; // show full header when hovering over the minimal header
        _this.mouseLeave = function () { return _this.header.classList.remove('show-header'); }; // hide header again
        _this._active = null;
        _this.node.dataset.layout = 'tabbing';
        _this.header.dataset.layout = 'tabbing';
        children.forEach(function (d) { return _this.push(d); });
        if (_this.options.active != null && _this.length >= _this.options.active) {
            _this.active = _this._children[_this.options.active];
        }
        if (!_this.options.fixedLayout) {
            dropAble(_this.header, [ALayoutContainer.MIME_TYPE], function (result) {
                var id = parseInt(result.data[ALayoutContainer.MIME_TYPE], 10);
                console.assert(id >= 0);
                //find id and move it here
                var root = _this.rootParent;
                var toMove = root.find(id);
                if (toMove === null || toMove === _this || toMove instanceof AParentLayoutContainer && _this.parents.indexOf(toMove) >= 0) {
                    //can't move parent into me
                    return false;
                }
                var alreadyChild = _this._children.indexOf(toMove) >= 0;
                if (alreadyChild) {
                    _this.moveChild(toMove, _this.length);
                }
                else {
                    //not a child already
                    _this.push(toMove);
                }
                return true;
            }, null, true);
        }
        if (_this.options.fixed) {
            _this.header.classList.add('fixed');
            _this.toggleFrozenLayout();
            _this.on(LayoutContainerEvents.EVENT_LAYOUT_CHANGED, function () {
                _this.toggleFrozenLayout();
            });
        }
        return _this;
    }
    TabbingLayoutContainer.prototype.canDrop = function (area) {
        return area === 'center';
    };
    TabbingLayoutContainer.prototype.place = function (child, reference, area) {
        console.assert(area === 'center');
        return this.push(child);
    };
    TabbingLayoutContainer.prototype.defaultOptions = function () {
        return Object.assign(_super.prototype.defaultOptions.call(this), {
            active: null
        });
    };
    Object.defineProperty(TabbingLayoutContainer.prototype, "active", {
        get: function () {
            return this._active;
        },
        set: function (child) {
            console.assert(!child || this._children.indexOf(child) >= 0);
            if (this._active === child) {
                return;
            }
            this.activeChanged(this._active, this._active = child);
        },
        enumerable: true,
        configurable: true
    });
    TabbingLayoutContainer.prototype.reorderAble = function (reorder) {
        var _this = this;
        dropAble(reorder, [ALayoutContainer.MIME_TYPE], function (result) {
            var id = parseInt(result.data[ALayoutContainer.MIME_TYPE], 10);
            console.assert(id >= 0);
            //find id and move it here
            var root = _this.rootParent;
            var toMove = root.find(id);
            if (toMove === null || toMove === _this || toMove instanceof AParentLayoutContainer && _this.parents.indexOf(toMove) >= 0) {
                //can't move parent into me
                return false;
            }
            //next sibling = managed header
            var index = _this._children.findIndex(function (d) { return d.header === reorder.nextSibling; });
            var alreadyChild = _this._children.indexOf(toMove) >= 0;
            if (alreadyChild) {
                _this.moveChild(toMove, index);
            }
            else {
                //not a child already
                _this.push(toMove, index);
            }
            return true;
        }, null, true);
    };
    TabbingLayoutContainer.prototype.addedChild = function (child, index) {
        var _this = this;
        _super.prototype.addedChild.call(this, child, index);
        child.visible = child === this.active;
        child.header.onclick = function () {
            _this.active = child;
        };
        this.header.insertAdjacentHTML('beforeend', TabbingLayoutContainer.TAB_REORDER);
        var reorder = this.header.lastElementChild;
        this.reorderAble(reorder);
        if (index < 0 || index >= this.length - 1) {
            this.header.appendChild(child.header);
            this.node.appendChild(child.node);
        }
        else {
            this.header.insertBefore(child.header, this._children[index + 1].header.previousSibling);
            this.header.insertBefore(reorder, child.header);
            this.node.insertBefore(child.node, this._children[index + 1].node);
        }
        if (this.active === null) {
            this.active = child;
        }
    };
    TabbingLayoutContainer.prototype.moveChild = function (child, index) {
        var old = this._children.indexOf(child);
        var atEnd = index === this.length;
        if (old === index || (atEnd && old === index - 1)) {
            //already at the right position
            return;
        }
        this._children.splice(old, 1);
        if (old < index) {
            index -= 1; //since we removed it already
        }
        this._children.splice(index, 0, child);
        //update header
        var reorder = child.header.previousSibling;
        if (atEnd) {
            //reorder
            this.header.appendChild(reorder);
            this.header.appendChild(child.header);
            this.node.appendChild(child.node);
            return;
        }
        var next = this._children[index + 1];
        this.header.insertBefore(child.header, next.header.previousSibling); //2 extra items
        this.header.insertBefore(reorder, child.header);
        this.node.insertBefore(child.node, next.node);
        this.fire(withChanged(LayoutContainerEvents.EVENT_TAB_REORDED), child, index);
    };
    TabbingLayoutContainer.prototype.replace = function (child, replacement) {
        var wasActive = child === this.active;
        _super.prototype.replace.call(this, child, replacement);
        if (wasActive) {
            this.active = replacement;
        }
        return true;
    };
    TabbingLayoutContainer.prototype.takeDownChild = function (child) {
        if (this.active === child) {
            var index = this._children.indexOf(child);
            this.active = this.length === 1 ? null : (index === 0 ? this._children[1] : this._children[index - 1]);
        }
        child.header.onclick = null;
        //reorder
        this.header.removeChild(child.header.previousSibling);
        this.header.removeChild(child.header);
        this.node.removeChild(child.node);
        _super.prototype.takeDownChild.call(this, child);
    };
    Object.defineProperty(TabbingLayoutContainer.prototype, "minSize", {
        get: function () {
            //max
            return this._children.reduce(function (a, c) {
                var cmin = c.minSize;
                return [Math.max(a[0], cmin[0]), Math.max(a[1], cmin[1])];
            }, [0, 0]);
        },
        enumerable: true,
        configurable: true
    });
    TabbingLayoutContainer.prototype.activeChanged = function (oldActive, newActive) {
        if (oldActive) {
            oldActive.header.classList.remove('active');
            oldActive.node.classList.remove('active');
            oldActive.visible = false;
        }
        if (newActive) {
            newActive.header.classList.add('active');
            newActive.node.classList.add('active');
            newActive.visible = this.visible;
        }
        this.fire(withChanged(LayoutContainerEvents.EVENT_CHANGE_ACTIVE_TAB), oldActive, newActive);
    };
    TabbingLayoutContainer.prototype.visibilityChanged = function (visible) {
        if (this.active) {
            this.active.visible = visible;
        }
    };
    TabbingLayoutContainer.prototype.persist = function () {
        return Object.assign(_super.prototype.persist.call(this), {
            type: 'tabbing',
            active: this._active ? this._children.indexOf(this._active) : null
        });
    };
    TabbingLayoutContainer.restore = function (dump, restore, doc) {
        var r = new TabbingLayoutContainer(doc, ALayoutContainer.restoreOptions(dump));
        dump.children.forEach(function (d) { return r.push(restore(d)); });
        if (r.active != null) {
            r.active = r.children[dump.active];
        }
        return r;
    };
    TabbingLayoutContainer.derive = function (node, derive) {
        var r = new TabbingLayoutContainer(node.ownerDocument, ALayoutContainer.deriveOptions(node));
        var children = Array.from(node.children);
        var activeIndex = children.findIndex(function (c) { return c.classList.contains('active'); });
        children.forEach(function (c) { return r.push(derive(c)); });
        if (activeIndex > 0) {
            r.active = r.children[activeIndex];
        }
        return r;
    };
    TabbingLayoutContainer.prototype.toggleFrozenLayout = function () {
        if (this.children.length < 2) { // frozen layout to apply minimal style to the header and hide views
            this.header.classList.add('floating-header');
            this.header.addEventListener('mouseenter', this.mouseEnter);
            this.header.addEventListener('mouseleave', this.mouseLeave);
        }
        else {
            this.header.classList.remove('floating-header');
            this.header.removeEventListener('mouseenter', this.mouseEnter);
            this.header.removeEventListener('mouseleave', this.mouseLeave);
        }
    };
    TabbingLayoutContainer.TAB_REORDER = "<div data-layout=\"tab-reorder\">&nbsp;</div>";
    return TabbingLayoutContainer;
}(AParentLayoutContainer));
export default TabbingLayoutContainer;
//# sourceMappingURL=TabbingLayoutContainer.js.map